/*

LCD circuit:
 * LCD RS pin to digital pin 7     (IC 13 - LCD  4)
 * LCD Enable pin to digital pin 6 (IC 12 - LCD  6)
 * LCD D4 pin to digital pin 5     (IC 11 - LCD 11)
 * LCD D5 pin to digital pin 4     (IC  6 - LCD 12)
 * LCD D6 pin to digital pin 3     (IC  5 - LCD 13)
 * LCD D7 pin to digital pin 2     (IC  4 - LCD 14)
 * LCD R/W pin to ground
 * 10K resistor:
 * ends to +5V and ground
 * wiper to LCD VO pin (pin 3)
 */


// include the library code:
#include <LiquidCrystal.h>

// initialize the library with the numbers of the interface pins
LiquidCrystal lcd(7, 6, 5, 4, 3, 2);

// Pin 13 has an LED connected on most Arduino boards.
int led = 13;
int i_led = 0;

// set input pin numbers:
const int buttonPin = 8;     // the number of the pushbutton pin
const int FanPin =  9;       // the number of the Fan pin
int FanOn = 1;               // Fan state flag
// variables will change:
int buttonState = 0;         // variable for reading the pushbutton status
int button_buf = 0;          // Button read state buffer
int disp = 0 ;               // display states    
                             // 0 : normal display,  Vp and Vm display
                             // 1 : temperature  display
    
//Temperature
float tempK = 0, tempC = 0;
int i_temp = 0;
double tempC_ave, tempC_ave_tmp = 0;

//Voltage
float Vp, Vp_ave, Vp_ave_tmp;  // V plus
float Vm, Vm_ave, Vm_ave_tmp;  // V minus
int i_v = 0;

void setup() {
  // set up the LCD's number of columns and rows: 
  lcd.begin(16, 2);
  lcd.clear();
  // Print a message to the LCD.
  lcd.print("Hello!");
  
  // initialize the digital pin as an output.
  pinMode(led, OUTPUT);   
  
   // initialize the Fan pin as an output:
  pinMode(FanPin, OUTPUT);      
  // initialize the pushbutton pin as an input:
  pinMode(buttonPin, INPUT);     

  //serial setting for debug
  //Serial.begin(9600);    //Setup serial to 9600 bps  

}

void loop() {
  
// LED Flush
  if (i_led<200){
    digitalWrite(led,  HIGH);
  }
  else {
    digitalWrite(led,  LOW);
    if (i_led>=400) i_led = 0;
  }
  i_led++;


// read the state of the pushbutton value:
  buttonState = digitalRead(buttonPin);
  if (buttonState == LOW) {
    button_buf++;
    if (button_buf >= 10) {
      button_buf = -1;
      disp ++ ;
      lcd.clear();
      if (disp == 2) disp = 0;
    }
  } else {
    button_buf = 0;
  }

//temperature
tempK = analogRead(5) * 0.004882812 * 100;    //Read temperature in Kelvins first
tempC = tempK - 273.15;    //Convert from Kelvin to Celsius

if (i_temp<1000){            // 1000 average
  tempC_ave_tmp = tempC_ave_tmp + tempC;
  i_temp++;
    }
  else {
    tempC_ave = tempC_ave_tmp / 1000;
    tempC_ave_tmp = 0;
    i_temp = 0;
    
  }

// Fan Control   25deg OFF / 30deg ON
if     (tempC_ave < 25)  FanOn = 0; 
else if(tempC_ave > 30)  FanOn = 1; 

if (FanOn == 0) digitalWrite(FanPin, LOW);
else            digitalWrite(FanPin, HIGH);  
 

//V+- measurement
Vp = analogRead(4) * 4.0 * 5 / 1024 + 0.06;    //Read temperature in Kelvins first
Vm = analogRead(3) * 4.0 * 5 / 1024 * (-1) - 0.06;    //Read temperature in Kelvins first

if (i_v < 100){            // 100 average
  Vp_ave_tmp = Vp_ave_tmp + Vp;
  Vm_ave_tmp = Vm_ave_tmp + Vm;
  i_v++;
    }
  else {
    Vp_ave = Vp_ave_tmp / 100;
    Vm_ave = Vm_ave_tmp / 100;
    Vp_ave_tmp = 0;
    Vm_ave_tmp = 0;
    i_v = 0;
  }


//    Serial.print("Vp: "); Serial.println(Vp);
//    Serial.print("Vm: "); Serial.println(Vm);
//    Serial.println();    //Print Blank Line

  if (disp == 0){
    lcd.setCursor(0, 0);
    lcd.print("V+ : +");
    lcd.print(Vp);
    lcd.print(" V    ");
    lcd.setCursor(0, 1);
    lcd.print("V- : ");
    lcd.print(Vm);
    lcd.print(" V    ");
  } else if (disp == 1){
    lcd.setCursor(0, 0);
    lcd.print("Temperature");
    lcd.setCursor(0, 1);
    lcd.print(tempC_ave);
  lcd.print(" deg    ");
  }
//delay(100);    //Delay 1 second

}

